#!/usr/bin/env python
"The rainbow server browser."

#    <one line to give the program's name and a brief idea of what it does.>
#    Copyright (C) 2008  Joakim Soderlund
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

import os
import gtk
import pyquake3
import urllib
import configobj

from time import sleep
from xml.dom import minidom
from threading import Thread
from SimpleGladeApp import SimpleGladeApp
from SimpleGladeApp import bindtextdomain

__author__    = "Joakim Soderlund"
__license__   = "GPLv2"
__copyright__ = "Copyright (C) 2008  Joakim Soderlund"
__date__      = "2008-07-10"
__version__   = "1.0"

cfgCompVers   = "1.0"
app_name      = "rnlauncher"
app_version   = "1.0"
glade_dir     = ""
locale_dir    = ""
serverListURL ="http://rainbow.furver.se/rainbow_server_list.xml"

bindtextdomain(app_name, locale_dir)
gtk.gdk.threads_init()

def loadConfig():
	if os.name == "nt":
		mw.config = configobj.ConfigObj("config.ini")
	else:
		if not os.path.isdir(os.path.join(os.path.expanduser("~"), ".rnlauncher")):
			os.mkdir(os.path.join(os.path.expanduser("~"), ".rnlauncher"))
		mw.config = configobj.ConfigObj(os.path.join(os.path.expanduser("~"), ".rnlauncher", "config.ini"))

	try:
		if mw.config["main"]["cfgVer"] != cfgCompVers:
			raise Exception("Invalid config version!")
	except:
		mw.config["main"] = {"cfgVer":"1.0", "refreshOnStartup":0, "oa071bin":"", "oa077bin":""}
		mw.config.write()
	if int(mw.config["main"]["refreshOnStartup"]):
		mw.refreshServers()

class threadedQuery(Thread):
	def __init__(self, address):
		Thread.__init__(self)
		self.address = address
	def run(self):
		try:
			address = self.address
			server = pyquake3.PyQuake3(address)
			server.update()
			mw.rnServerDict[address] = server
	
			if server.vars["protocol"] == "69" and not os.path.isfile(mw.config["main"]["oa077bin"]):
					pass
			elif server.vars["protocol"] == "68" and not os.path.isfile(mw.config["main"]["oa071bin"]):
					pass
			else:					
				name = server.vars["sv_hostname"]
				name = name.strip("|http://sv.furver.se")
				name = name.strip()
				if name.startswith("RN|^"):
					name = name[7:]
					name = name.strip()
					name = "RN|" + name
				name = name.replace("-", "")
				name = name.replace("^", "")
				name = name.replace("*", " ")
				for a in range(10):
					name = name.replace("  ", " ")
				name = name.strip()
				if name.startswith("RN|"):
					name = name[3:]
					name = name.strip()
					name = "RN|" + name
					mw.serverList.append([name, server.vars["mapname"], "%02d/%02d" % (len(server.players), int(server.vars["sv_maxclients"])), address])
		except Exception:
			pass

class getServerInfo(Thread):
	"Updates the server-list."
	def __init__(self):
		Thread.__init__(self)

	def run(self):
		mw.btn_refreshServers.set_sensitive(0)
		mw.serverList.clear()
		if not os.path.isfile(mw.config["main"]["oa077bin"]) and not os.path.isfile(mw.config["main"]["oa077bin"]):
			mw.statusbar.push(0, "No OpenArena binaries found! Check your settings!")
			sleep(10)
			mw.btn_refreshServers.set_sensitive(1)
			mw.statusbar.push(0, " ")
			return
		mw.statusbar.push(0, "Retrieving server list from furver.se...")
		xmldata = urllib.urlopen(serverListURL).read()
		xmldoc = minidom.parseString(xmldata)
		rnServerList = []		
		for server in xmldoc.getElementsByTagName("server"):
			if server.getElementsByTagName("type")[0].firstChild.toxml() == "quake3":
				rnServerList.append(	server.getElementsByTagName("address")[0].firstChild.toxml() + ":" +
									server.getElementsByTagName("port")[0].firstChild.toxml())

		mw.statusbar.push(0, "Reciving server info from %s servers..." % len(rnServerList))
		mw.rnServerDict = {}
		for address in rnServerList:
			self.threadedQuery = threadedQuery(address)
			self.threadedQuery.start()
		mw.btn_connect.set_sensitive(1)
		mw.btn_serverProperties.set_sensitive(1)
		sleep(3)
		mw.statusbar.push(0, " ")
		mw.btn_refreshServers.set_sensitive(1)

class runOpenArena(Thread):
	def __init__(self, connectionString):
		Thread.__init__(self)
		self.connectionString = connectionString
	def run(self):
		os.popen(self.connectionString)

class runMessages(Thread):
	def __init__(self):
		Thread.__init__(self)
	def run(self):
		mw.btn_connect.set_sensitive(0)
		mw.btn_serverProperties.set_sensitive(0)
		mw.statusbar.push(0, "Launching OpenArena...")
		sleep(10)
		mw.statusbar.push(0, " ")
		mw.btn_connect.set_sensitive(1)
		mw.btn_serverProperties.set_sensitive(1)

class mainWindow(SimpleGladeApp):
	def __init__(self, path="rnlauncher.glade", root="mainWindow", domain=app_name, **kwargs):
		path = os.path.join(glade_dir, path)
		SimpleGladeApp.__init__(self, path, root, domain, **kwargs)

		self.addServerListColumn("Server name", 0)
		self.addServerListColumn("Map", 1)
		self.addServerListColumn("Players", 2)
		self.addServerListColumn("Address", 3)

		self.serverList = gtk.ListStore(str, str, str, str)
		self.serverView.set_model(self.serverList)

		self.statusbar.push(0, " ")

	def addServerListColumn(self, colTitle, colID):
		"Creats columns for the serverView."
		column = gtk.TreeViewColumn(colTitle, gtk.CellRendererText(), text=colID)
		column.set_resizable(True)
		column.set_sort_column_id(colID)
		self.serverView.append_column(column)
		
	def on_btn_refreshServers_clicked(self, widget, *args):
		self.refreshServers()

	def refreshServers(self):
		self.getServerInfo = getServerInfo()
		self.getServerInfo.start()

	def getSelectedIP(self):
		rowNum = int("%s" % self.serverView.get_selection().get_selected_rows()[1][0])
		item = self.serverList.get_iter_first()
		cats = []
		while ( item != None ):
			cats.append(self.serverList.get_value (item, 3))
			item = self.serverList.iter_next(item)
		return cats[rowNum]

	def connectToServer(self):
		serverIP = self.getSelectedIP()

		if self.rnServerDict[serverIP].vars["protocol"] == "69":
			oabin = self.config["main"]["oa077bin"]
		elif self.rnServerDict[serverIP].vars["protocol"] == "68":
			oabin = self.config["main"]["oa071bin"]
		else:
			raise Exception("Error! Launched incompatible protocol version!")

		try:
			rconpassword = self.config[serverIP]["rconpassword"]
		except:
			rconpassword = ""
		self.runOpenArena = runOpenArena("%s +connect \"%s\" +set rconpassword \"%s\"" % (oabin, serverIP, rconpassword))
		self.runMessages = runMessages()
		self.runMessages.start()
		self.runOpenArena.start()


	def on_btn_connect_clicked(self, widget, *args):
		self.connectToServer()

	def on_btn_serverProperties_clicked(self, widget, *args):
		serverIP = self.getSelectedIP()
		self.serverProperties = serverSettings()
		try:
			rconpassword = self.config[serverIP]["rconpassword"]
		except:
			rconpassword = ""
		self.serverProperties.recieveServerInfo(serverIP, self.rnServerDict[serverIP], rconpassword)
		self.serverProperties.run()

	def on_btn_mainSettings_clicked(self, widget, *args):
		self.mainSettings = mainSettings()
		self.mainSettings.run()

	def on_serverView_row_activated(self, widget, *args):
		self.connectToServer()

	def on_menu_about_activate(self, widget, *args):
		aboutdialog = aboutDialog()
		aboutdialog.run()
		aboutdialog.destroy()

	def on_mainWindow_destroy(self, widget, *args):
		gtk.main_quit()

	def run(self):
		gtk.gdk.threads_enter()
		gtk.main()
		gtk.gdk.threads_leave()

class aboutDialog(SimpleGladeApp):
	def __init__(self, path="rnlauncher.glade", root="aboutDialog", domain=app_name, **kwargs):
		path = os.path.join(glade_dir, path)
		SimpleGladeApp.__init__(self, path, root, domain, **kwargs)

	def on_aboutDialog_destroy(self, widget, *args):
		gtk.main_quit()

class mainSettings(SimpleGladeApp):
	def __init__(self, path="rnlauncher.glade", root="mainSettings", domain=app_name, **kwargs):
		path = os.path.join(glade_dir, path)
		SimpleGladeApp.__init__(self, path, root, domain, **kwargs)
		self.ent_oa071bin.set_text(mw.config["main"]["oa071bin"])
		self.ent_oa077bin.set_text(mw.config["main"]["oa077bin"])
		self.cbx_refreshOnStartup.set_active(int(mw.config["main"]["refreshOnStartup"]))

	def on_btn_ok2_clicked(self, widget, *args):
		mw.config["main"]["oa071bin"] = self.ent_oa071bin.get_text()
		mw.config["main"]["oa077bin"] = self.ent_oa077bin.get_text()
		if self.cbx_refreshOnStartup.get_active():
			mw.config["main"]["refreshOnStartup"] = 1
		else:
			mw.config["main"]["refreshOnStartup"] = 0
		mw.config.write()
		self.mainSettings.destroy()

	def on_btn_oa071bin_clicked(self, widget, *args):
		self.fileChooser = fileChooser()
		self.fileChooser.fileChooser.set_title("Select your Open Arena 0.7.1 binary")
		self.fileChooser.run()
		try:
			self.ent_oa071bin.set_text(self.fileChooser.filename)
		except AttributeError:
			pass

	def on_btn_oa077bin_clicked(self, widget, *args):
		self.fileChooser = fileChooser()
		self.fileChooser.fileChooser.set_title("Select your Open Arena 0.7.7 binary")
		self.fileChooser.run()
		try:
			self.ent_oa077bin.set_text(self.fileChooser.filename)
		except AttributeError:
			pass

	def on_btn_abort2_clicked(self, widget, *args):
		self.mainSettings.destroy()

	def on_mainSettings_destroy(self, widget, *args):
		gtk.main_quit()



class serverSettings(SimpleGladeApp):
	def __init__(self, path="rnlauncher.glade", root="serverSettings", domain=app_name, **kwargs):
		path = os.path.join(glade_dir, path)
		SimpleGladeApp.__init__(self, path, root, domain, **kwargs)


	def recieveServerInfo(self, serverIP, serverData, rconpassword):
		self.serverIP = serverIP
		self.serverData = serverData
		self.lab_serverName.set_text(serverIP)
		self.ent_rconPassword.set_text(rconpassword)

	def on_btn_ok_clicked(self, widget, *args):
		try:
			mw.config[self.serverIP]["rconpassword"] = self.ent_rconPassword.get_text()
		except KeyError:
			mw.config[self.serverIP] = {}
			mw.config[self.serverIP]["rconpassword"] = self.ent_rconPassword.get_text()
		finally:
			mw.config.write()
			self.serverSettings.destroy()
		
	def on_btn_abort_clicked(self, widget, *args):
		self.serverSettings.destroy()

	def on_serverSettings_destroy(self, widget, *args):
		gtk.main_quit()

class fileChooser(SimpleGladeApp):
	def __init__(self, path="rnlauncher.glade", root="fileChooser", domain=app_name, **kwargs):
		path = os.path.join(glade_dir, path)
		SimpleGladeApp.__init__(self, path, root, domain, **kwargs)

	def on_btn_okfc_clicked(self, widget, *args):
		self.filename = self.fileChooser.get_filename()
		self.fileChooser.destroy()
		gtk.main_quit()

	def on_btn_abortfc_clicked(self, widget, *args):
		self.fileChooser.destroy()
		gtk.main_quit()

	def on_fileChooser_destroy(self, widget, *args):
		gtk.main_quit()

if __name__ == "__main__":
	mw = mainWindow()
	loadConfig()
	mw.run()
	mw.config.write()
